#!/bin/sh
# The next line is executed by /bin/sh, but not Tcl \
    exec /usr/local/bin/wish8.3 $0 ${1+"$@"}

# if you are installing by hand, 
# replace /usr/local/bin/wish8.3
# on line 3 with the full path to your wish executable, verison 8.3 or higher.

global startupdir
set startupdir ""
set scriptname [info script]
# on unix, chase symbolic links until we get to the real program
if {"$tcl_platform(platform)" == "unix"} {
    while {[catch {set scriptname [file readlink $scriptname]}] == 0} { }
}
set startupdir [file dirname $scriptname]
if {"$startupdir" == "."} {
     set startupdir [pwd]
}
lappend auto_path $startupdir [file dirname $startupdir]

package require mkWidgets
package require msTclObjs
package require Mapscript

option add *Dialog.msg.wrapLength 700
option add *Dialog.msg.font {helvetica 12 bold}

global title
set title "MapServer App Wizard"

global mapobj	
set mapobj  ""

global mapfile
set mapfile "" 

global symobj	
set symobj  ""

global webserverhome
set webserverhome ""

global mapapphome
set mapapphome ""

global mapcgi
set mapcgi ""

global html
set html ""

global imgtype
set imgtype ""

global tmploc tmpdir
set tmploc ""
set tmpdir ""

global querydefs
set querydefs ""

global all_layers
set all_layers ""

global initial
set initial ""

global nextCmd prevCmd 
set nextCmd ""
set prevCmd ""

global lastDataPath
set lastDataPath ""

proc appErr {msg} {
    global title
    tk_messageBox -title $title -type ok -icon error \
	-message $msg
}

proc loadSymbols {} {
    global startupdir symobj allsym
    set lexer [mstclobj::mapLexer #auto]
    $lexer initFromFile $startupdir/appwizlib/all.sym
    set symobj [mstclobj::symbolsetTclObj #auto]
    $symobj parse $lexer
    ::obstcl::delete $lexer
    set allsym [list (none)]
    foreach c [$symobj getSubObjs] {
	set sym_name [$c getAttr name]
        if {[string length $sym_name] && "$sym_name" != "#none"} {
	    lappend allsym $sym_name
        }
    }
}

proc doPrevious {} {
    global prevCmd
    if {[string length $prevCmd]} {
        eval $prevCmd
    }
}

proc doNext {} {
    global nextCmd
    if {[string length $nextCmd]} {
        eval $nextCmd
    }
}

proc doCancel {} {
    global title
    set res [tk_messageBox -title $title -type okcancel \
	-icon warning -message "Your application is incomplete.\n\n\
                            You do wish to quit?" ]

    if {"$res" == "ok"} {
        exit
    }
}


################################################################################
# welcome
#

proc welcome {} {
    global nextCmd prevCmd mapobj
    set w .top.welcome
    if {![winfo exists $w]} {
	frame $w -width 550 -height 350
	label $w.wel -text "\nWelcome to MapServer App Wizard\n\n\
                    Tom Poindexter\ntpoindex@nyx.net\nCopyright 2000\n\n\
                    press \"Next\" to begin building your application."
	pack $w.wel -side top -padx 5 -pady 5 
    }
    pack forget [pack slaves .top]
    pack $w
    set nextCmd askName
    set prevCmd ""
    if {[string length $mapobj] ==0} {
	set mapobj [mstclobj::mapTclObj #auto]
    }
}

# welcome
#
################################################################################


################################################################################
# askName
#

proc askName {} {
    global nextCmd prevCmd mapobj
    .bot.prev configure -state active

    set w .top.askname
    if {![winfo exists $w]} {
	frame $w -width 550 -height 350
	label $w.m -text "\nApplication name\n\nYour MapServer application\
              requires a name,\nwhich will also create a new directory\n\
              of the same name in your current directory."
	label $w.pwd -text "Current directory:\n[pwd]"
	button $w.cd -text "change directory" -command askNameCD \
		-font {helvetica 10}
	frame $w.f
        label $w.f.l -text "Application name:"
	entry $w.f.e -width 15
	bind $w.f.e <KeyPress-Return> doNext
	bind $w.f.e <KeyPress-Tab>    doNext
	pack  $w.f.l $w.f.e -side left -anchor e
	pack $w.m $w.pwd $w.cd $w.f -side top -padx 5 -pady 5
    }
    pack forget [pack slaves .top]
    pack $w
    $w.f.e delete 0 end
    set name [$mapobj getAttr name]
    if {"$name" == "#none"} {
	set name ""
    }
    $w.f.e insert 0 $name
    set nextCmd askNameDO
    set prevCmd welcome
}
proc askNameCD {} {
    set w .top.askname
    set newdir [tk_chooseDirectory]
    # windows sometimes puts our window below other after tk_chooseDirectory
    raise .
    if {[string length $newdir]} {
        cd $newdir
	$w.pwd configure -text "Current directory:\n[pwd]"
    }
}
proc askNameDO {} {
    global nextCmd prevCmd mapobj title
    set w .top.askname
    set name [string trim [$w.f.e get]]
    if {![string length $name]} {
	appErr  "Please enter an application name."
	return
    }
    if {[regexp {[ /\\\\.]} $name]} {
	appErr  "Your name cannot contain blanks, slashes, or periods."
	return
    }
    if {[file isfile $name]} {
	appErr  "\"$name\" is already a file name.\
                  \n\nPlease choose a different application name."
	return
    }
    if {[file isdirectory $name]} {
	if {[file isfile $name/$name.map] || ( \
	    [file isdirectory $name/data] && \
	    [file isdirectory $name/symbols] && \
	    [file isdirectory $name/images] ) } {
	    set res [tk_messageBox -icon warning -type okcancel \
               -title $title -message "A MapServer application\
                 of \"$name\" already exists.\n\nOverwrite existing map\
                 application files?" ]
	    if {"$res" == "cancel"} {
                appErr "\"$name\" is already a map\
                   application.\n\nPlease choose a different name."
		return
	    }
        } else {
	    appErr  "\"$name\" is an existing directory name.\
                   \n\nPlease choose a different application name."
	    return
	}
    }

    $mapobj setAttr name $name
    # will set  extent and units later

    if { [file isdirectory $name] } {
        setMsg "directory $name already exists"
    } else {
	if { [catch {file mkdir $name} res]} {
	    tk_messageBox -type ok -icon error -title $title \
		-message "Couldn't create new directory \"$name\"\n\nReason:\n\
                  $res"
	    setMsg "directory not created"
            return
	}
        setMsg "directory $name created"
    }
    cd $name
    catch {file mkdir data}
    catch {file mkdir graphics}
    catch {file mkdir symbols}
    catch {file mkdir symbols/images}
    catch {file mkdir images}
    catch {file mkdir tmp}
    gatherData
}

# askName
#
################################################################################

################################################################################
# gatherData
#
proc gatherData {} {
    global nextCmd prevCmd mapobj
    set w .top.gatherdata
    if {![winfo exists $w]} {
	frame $w -width 550 -height 350
	label $w.m -text "\nGet Map data\n\nGather that data sources that will\
             become layers in your map.\nValid data files are ESRI Shapfile\
             (.shp) and AcrSDE (.sde),\nTiFF/GeoTiFF (.tiff), GIF (.gif),\
              PNG *.png), ERDAS, JPEG (.jpg), and EPPL7.\nAll data files must\
              be in the same projection system."

	frame $w.u
        label $w.u.units_l -text "Map units:"
	combobox $w.u.units -state restricted -entries \
		"feet inches kilometers meters miles dd"
	$w.u.units see 3
	pack $w.u.units_l $w.u.units -side left 

        frame $w.s
	label $w.s.scale_l -text "Scale bar units:"
	combobox $w.s.scale -state restricted -entries \
		"feet inches kilometers meters miles"
        $w.s.scale see 3
	pack  $w.s.scale_l $w.s.scale -side left

	label $w.m1 -text "current data files:"
	scrollbox $w.lb -scrollbar on -height 5 -width 35
	button $w.b -text "Add data file..." -command gatherDataGET \
		-font {helvetica 10}
	pack $w.m $w.u $w.s $w.m1 $w.lb $w.b -side top -padx 5 -pady 4
    }
    pack forget [pack slaves .top]
    pack $w
    $w.lb delete 0 end
    foreach f [glob -nocomplain data/*] {
	$w.lb insert end $f
    }
    set nextCmd gatherDataDO
    set prevCmd askName
}
proc gatherDataGET {} {
    global lastDataPath
    setMsg ""
    set w .top.gatherdata
    set ftypes {{All *} {Symbol .sym} {Font .fnt} {Dbase .dbx} \
		{Shape .shp}  {Gif .gif} {Jpg .jp*g} {Tiff .tif*} {Png *.png} }
    set shp 0
    set shpmsg ""
    if {[string length $lastDataPath]} {
	set init_dir $lastDataPath
    } else {
	set init_dir [file dirname [pwd]]
    }
    set fn [tk_getOpenFile -filetypes $ftypes -initialdir $init_dir]
    if {[string length $fn]} {
	if {"[file dirname $fn]" == "[file join [pwd] data]"} {
	    appErr " You cannot choose a file already in\n\
		the data directory.  Please choose another file."
	    return
	}
        if {[file isfile $fn]} {
	    if {[regexp -nocase .shp [file extension $fn]]} {
		set shp 1
		set shpmsg "\nThis shapefile's associated .shx\
                      and .dbf files\nwill also be copied or moved.\n"
	    }
	    set res [tk_dialog .t "MapServer App Wizard" " [file tail $fn]\n\n\
               Do you wish to copy this file,\n\
               or move it (removing it from it's current location)?\n$shpmsg" \
               "" 0 Copy Move Cancel]
            if {$res == 2 || $res == -1} {
		return
            }
	    if {$shp} {
		set shxpat "[file rootname $fn].\[Ss\]\[Hh\]\[Xx\]"
		set dbfpat "[file rootname $fn].\[Dd\]\[Bb\]\[Ff\]"
		foreach other [glob -nocomplain $shxpat $dbfpat] {
		    lappend fn $other
		}
	    }
	    if {$res == 0} {
		foreach f $fn {
	            file copy -force -- $f [file join [pwd] data]
		}
	    } elseif {$res == 1} {
		foreach f $fn {
	            file rename -force -- $f [file join [pwd] data]
		}
	    }
 	    set lastDataPath [file dirname $f]

            setMsg "added [file tail $fn] to data"

	    $w.lb delete 0 end
	    foreach f [glob -nocomplain data/*] {
		$w.lb insert end $f
	    }
	    
	} else {
            setMsg ""
	    appErr "\"$fn\" cannot be found.\n\
                  Please choose another file."
	}
    }
}
proc gatherDataDO {} {
    global nextCmd prevCmd mapobj
    set w .top.gatherdata
    if {[llength [glob -nocomplain data/*]] == 0} {
	appErr  "No data files have been selected.\n\n\
                  Please choose data files for your application."
	return
    }

    # set map units
    $mapobj setAttr units [$w.u.units get]

    # define scalebar object
    set scaleobj [mstclobj::scalebarTclObj #auto $mapobj]
    $scaleobj setAttr imagecolor "255 255 255"
    $scaleobj setAttr style 0
    $scaleobj setAttr size "200 7"
    $scaleobj setAttr color "255 255 255"
    $scaleobj setAttr backgroundcolor "255 255 0"
    $scaleobj setAttr outlinecolor "0 0 0"
    $scaleobj setAttr units [$w.s.scale get]
    $scaleobj setAttr intervals 4
    $scaleobj setAttr transparent true
    $scaleobj setAttr status on
    set labobj [mstclobj::labelTclObj #auto $scaleobj]
    $labobj setAttr backgroundcolor "0 0 0"
    $labobj setAttr color "255 255 0"
    $labobj setAttr size tiny

    setMsg ""
    defineLayers
}

#
# gatherData
################################################################################


################################################################################
# defineLayers
#
proc defineLayers {} {
    global nextCmd prevCmd mapobj
    set w .top.definelayers
    if {![winfo exists $w]} {
	frame $w -width 550 -height 350
	label $w.m -text "\nDefine map Layers\n\nDefine map layers based on\
             the data sources.\nLayers you define will be rendered in the\
             order in which you define them.\nAdditionally, the first layer\
             defined will become the reference map image."
	label $w.m1 -text "current layers:"
	scrollbox $w.lb -scrollbar on -height 7 -width 20
        frame $w.b
	button $w.b.add -text "Add layer..." -command defineLayersNEW \
		-font {helvetica 10}
        button $w.b.del -text "Delete layer" -command deleteLayer \
		-font {helvetica 10}
        pack $w.b.add $w.b.del -side left -padx 8
	pack $w.m $w.m1 $w.lb $w.b -side top -padx 5 -pady 5
    }
    pack forget [pack slaves .top]
    pack $w
    defineLayersGetLayer
    set nextCmd defineLayersDO
    set prevCmd gatherData
}
proc deleteLayer {} {
    global mapobj
    set w .top.definelayers
    set idx [$w.lb curselection] 
    if {[string length $idx]} {
	set sel [$w.lb get $idx]
	set res [tk_dialog .t "MapServer App Wizard" "Do you wish to\
	   delete this layer?\n\n$sel\n" \
	   "" 0 Ok Cancel]
	if {"$res" == "0"} {
	    # get layer objects
	    set all ""
	    foreach l [$mapobj getSubObjs] {
		if {"[$l getType]" == "layer"} {
		    lappend all $l
		}
	    }
	    if {[llength $all] >= $idx} {
		set l [lindex $all $idx]
		::obstcl::delete $l
		setMsg "layer \"$sel\" deleted"
	    }
	    defineLayersGetLayer
	}
    } else {
	appErr "You must select a layer to delete"
    }
}
proc defineLayersGetLayer {} {
    global mapobj
    set w .top.definelayers
    $w.lb delete 0 end
    foreach l [$mapobj getSubObjs] {
	if {"[$l getType]" == "layer"} {
	    $w.lb insert end "[$l getAttr name]  ([$l getAttr data])"
	}
    }
   
}
proc defineLayersSelFile {scrl fname lname type classi labeli queryi} {
    array set types {
       1 point
       3 arc
       5 polygon
       8 multipoint
    }

    set idx [lindex [$scrl curselection] 0]
    if {[string length $idx]} {
	set fn [$scrl get $idx]
    } else {
	return
    }
    $lname delete 0 end
    $lname insert 0 [file rootname $fn]
    if {"[string tolower [file extension $fn]]" == ".shp"} {
        $fname configure -text [file rootname $fn]
	# set type from shapefile type
	# get datafields for classitem combobox
        set rc [mapscript::shapefileObj shp data/$fn -1]	
	if {"$rc" == "NULL"} {
	    $classi configure -state restricted -entries ""
	    $labeli configure -state restricted -entries ""
	} else {
	    switch $types([shp cget -type]) {
		point    {$type see 0} 
		arc      {$type see 1}
		polyline {$type see 2}
		polygon  {$type see 3}
            }
	    set hdbf [shp cget -hDBF]
	    if {"$hdbf" != "NULL"} {
	 	set dbf_names [mapscript::getDBFNames $hdbf]
	        $classi configure  -entries $dbf_names
	        $classi see 0
	        $labeli configure  -entries $dbf_names
	        $labeli see 0
                $queryi delete 0 end
		foreach n $dbf_names {
		    $queryi insert end $n
		}
	    } else {
		$classi configure -entries ""
		$labeli configure -entries ""
                $queryi delete 0 end
	    }
	}
	rename shp {}
    } else {
        $fname configure -text $fn
	# check for known raster type
	# set type as raster
	$type configure see 5
	$classi configure -entries ""
	$labeli configure -entries ""
        $queryi delete 0 end
    }
}
proc defineLayersNEW {} {
    global nextCmd prevCmd mapobj querydefs title
    global allsym
    global layerOk
    set layerOk ""

    setMsg ""

    catch {destroy .layer}
    toplevel .layer
    wm title .layer "Define Layer"
    
    # build layer window
    set w .layer

    label $w.data_l -text "Select a data source"
    scrollbox $w.data -scrollbar on -height 5 -width 20
    bind $w.data.list <Double-1> "+ $w.sel_data invoke"
    foreach fn [glob -nocomplain data/*] {
	set fn [file tail $fn]
	if {[regexp -nocase ".dbf|.shx" [file extension $fn]]} {
	    #ignore shapefile index and dbf files
	    continue
	}
	$w.data insert end $fn
    }
    button $w.sel_data -text "Select file" -font {helvetica 10} \
	-command "defineLayersSelFile $w.data $w.fn $w.name $w.type $w.classi \
	          $w.labeli $w.queryitems"

    grid $w.data_l   -column 0 -row 0 -sticky s -pady 4
    grid $w.data     -column 0 -row 1 -sticky news -rowspan 6 -padx 4
    grid $w.sel_data -column 0 -row 7 -sticky n -pady 4 


    label $w.exist_l -text "Existing layer names"
    scrollbox $w.exist -scrollbar on -height 5 -width 20
    bind $w.exist.list <Double-1> "+ $w.sel_lay invoke"
    foreach l [$mapobj getSubObjs] {
        if {"[$l getType]" == "layer"} {
            $w.exist insert end [$l getAttr name]
        }

    }
    button $w.sel_lay -text "Re-use layer name" -font {helvetica 10} \
	-command "\
	set tmpidx \[ $w.exist curselection \] ;             \
	if {\[ string length \$tmpidx \]}  \"                \
	     $w.name delete 0 end ;                          \
             $w.name insert 0 \[ $w.exist get \$tmpidx \]    \
	\"
    "
    grid $w.exist_l -column 0 -row 8 -sticky s -pady 4
    grid $w.exist   -column 0 -row 9 -sticky news -rowspan 6 -padx 4
    grid $w.sel_lay -column 0 -row 15 -sticky n -pady 4


    label $w.fn_l  -text "    Data file:"
    label $w.fn    -text ""
    label $w.name_l -text "    Layer name:"
    entry $w.name -width 20 -bg white
    grid $w.fn_l   -column 1 -row 0 -sticky e
    grid $w.fn     -column 2 -row 0 -sticky w -padx 4
    grid $w.name_l -column 1 -row 1 -sticky e
    grid $w.name   -column 2 -row 1 -sticky w -padx 4
    
 
    label $w.type_l -text "Type:"
    combobox $w.type -state restricted \
		-entries "point line polyline polygon annotation raster"
    grid $w.type_l -column 1 -row 2 -sticky e
    grid $w.type   -column 2 -row 2 -sticky w -padx 4

    label $w.classi_l -text "Class item:"
    combobox $w.classi -state restricted -entries ""
    grid $w.classi_l -column 1 -row 3 -sticky e
    grid $w.classi   -column 2 -row 3 -sticky w -padx 4
    
    label $w.labeli_l -text "Label item:"
    combobox $w.labeli -state restricted -entries ""
    grid $w.labeli_l -column 1 -row 4 -sticky e
    grid $w.labeli   -column 2 -row 4 -sticky w -padx 4
    
    label $w.queryopt_l -text "Query:"
    checkbutton $w.queryopt -text "Include this layer"
    set ::queryopt 0
    label $w.queryitems_l -text "Query result  \ntable fields:"
    scrollbox $w.queryitems -scrollbar on -selectmode multiple \
			-exportselection no -height 5 
    grid $w.queryopt_l   -column 1 -row 6 -sticky e
    grid $w.queryopt     -column 2 -row 6 -sticky w -padx 4
    grid $w.queryitems_l -column 1 -row 7 -sticky e -rowspan 6
    grid $w.queryitems   -column 2 -row 7 -sticky w -padx 4 -rowspan 6
    
    label $w.class1opt_l -text "1st Class Definition"
    checkbutton $w.class1opt -text Active 
    set ::class1opt 1
    label $w.class1exp_l -text \
		"Class expression:"
    entry $w.class1exp -width 20 -bg white
    entry $w.class1_r 
    entry $w.class1_g
    entry $w.class1_b
    $w.class1_r insert 0 128
    $w.class1_g insert 0 128
    $w.class1_b insert 0 128
    button $w.class1color -font {helvetica 10} -text "Choose color..." \
	-command "chooseColor $w.class1_r $w.class1_g $w.class1_b; raise .layer"
    grid $w.class1opt_l  -column 3 -row 1 -sticky e
    grid $w.class1opt    -column 4 -row 1 -sticky w
    grid $w.class1exp_l  -column 3 -row 2 -sticky e
    grid $w.class1exp    -column 4 -row 2 -sticky w -padx 4
    # colors are not shown
    grid $w.class1color  -column 4 -row 3 
 
    label $w.class1lab_l -text "Label size:"
    combobox $w.class1lab -state restricted -entries \
        "(none) tiny small medium large giant"
    $w.class1lab see 0
    label $w.class1pos_l -text "Label position:"
    combobox $w.class1pos -state restricted -entries \
	"auto ul uc ur cl cc cr ll lc lr"
    $w.class1pos see 0
    grid $w.class1lab_l -column 3 -row 4 -sticky e
    grid $w.class1lab   -column 4 -row 4 -sticky w -padx 4
    grid $w.class1pos_l -column 3 -row 5 -sticky e
    grid $w.class1pos   -column 4 -row 5 -sticky w -padx 4

    label $w.class1sym_l -text "Symbol:"
    combobox $w.class1sym -state restricted -entries $allsym
    grid $w.class1sym_l -column 3 -row 6 -sticky e
    grid $w.class1sym   -column 4 -row 6 -sticky w -padx 4


    label $w.class2opt_l -text "2nd Class Definition"
    checkbutton $w.class2opt -text Active 
    set ::class2opt 0
    label $w.class2exp_l -text \
		"Class expression:"
    entry $w.class2exp -width 20 -bg white
    entry $w.class2_r 
    entry $w.class2_g
    entry $w.class2_b
    $w.class2_r insert 0 128
    $w.class2_g insert 0 128
    $w.class2_b insert 0 128
    button $w.class2color -font {helvetica 10} -text "Choose color..." \
	-command "chooseColor $w.class2_r $w.class2_g $w.class2_b; raise .layer"
    grid $w.class2opt_l  -column 3 -row 10 -sticky e
    grid $w.class2opt    -column 4 -row 10 -sticky w
    grid $w.class2exp_l  -column 3 -row 11 -sticky e
    grid $w.class2exp    -column 4 -row 11 -sticky w -padx 4
    # colors are not shown
    grid $w.class2color  -column 4 -row 12
 
    label $w.class2lab_l -text "Label size:"
    combobox $w.class2lab -state restricted -entries \
        "(none) tiny small medium large giant"
    $w.class2lab see 0
    label $w.class2pos_l -text "Label position:"
    combobox $w.class2pos -state restricted -entries \
	"auto ul uc ur cl cc cr ll lc lr"
    $w.class2pos see 0
    grid $w.class2lab_l -column 3 -row 13 -sticky e
    grid $w.class2lab   -column 4 -row 13 -sticky w -padx 4
    grid $w.class2pos_l -column 3 -row 14 -sticky e
    grid $w.class2pos   -column 4 -row 14 -sticky w -padx 4

    label $w.class2sym_l -text "Symbol:"
    combobox $w.class2sym -state restricted -entries $allsym
    grid $w.class2sym_l -column 3 -row 15 -sticky e
    grid $w.class2sym   -column 4 -row 15 -sticky w -padx 4



    button $w.ok -text Ok      -command defineLayersOK 
    button $w.can -text Cancel -command defineLayersCANCEL
    grid $w.ok  -column 1 -row 15 -sticky e -padx 8 -pady 6
    grid $w.can -column 2 -row 15 -sticky w -padx 8 -pady 6

    # wait for user interaction
    grab $w
    tkwait variable layerOk
    
    # check for good values
    if {"$layerOk" == "ok"} {
	set layname [string trim [$w.name get]]
        set datafile [$w.fn cget -text]

        if {! [string length $datafile]} {
	    tk_messageBox -icon warning -type ok -title $title \
		-message "You have not choosen a date file for this layer."
	    return
        }

        if {! [string length $layname] } {
	    set layname [file rootname $datafile]
	    setMsg "layer name was not defined, using \"$layname\""
	}

	set layerobj [mstclobj::layerTclObj #auto $mapobj]
	$layerobj setAttr name  $layname
	$layerobj setAttr group $layname
        set classitem [string trim [$w.classi get]]
	$layerobj setAttr classitem      [string trim [$w.classi get]]
	$layerobj setAttr queryitem      [string trim [$w.classi get]]
	$layerobj setAttr data           $datafile
        if {[string length [string trim [$w.labeli get]]]} {
	    $layerobj setAttr labelitem  [string trim [$w.labeli get]]
	}
	$layerobj setAttr type           [string trim [$w.type get]]
	$layerobj setAttr labelcache     on
	$layerobj setAttr postlabelcache true
	$layerobj setAttr status         on

	# check for query data fields
	if {$::queryopt} {
            $layerobj setAttr header [file rootname $datafile]_header.html
            $layerobj setAttr footer [file rootname $datafile]_footer.html
	    set queryobj [mstclobj::queryTclObj #auto $layerobj]
            $queryobj setAttr template [file rootname $datafile]_query.html
	    set items ""
	    foreach idx [$w.queryitems curselection] {
		lappend items [$w.queryitems get $idx]
	    }
	    lappend querydefs [list $layerobj $queryobj $items]
	}
        
	# get class definitions for class1 and class2
	foreach n {1 2} {
	    if {[set ::class${n}opt]} {
		set classobj [mstclobj::classTclObj #auto $layerobj]
		if {[string length [$w.class${n}exp get]]} {
		    $classobj setAttr expression [$w.class${n}exp get]
		}
		$classobj setAttr name "[$w.name get]_$n"
		$classobj setAttr color [list [$w.class${n}_r get] \
				[$w.class${n}_g get] [$w.class${n}_b get]]
		$classobj setAttr outlinecolor    "0 0 0"
		$classobj setAttr backgroundcolor "128 128 128"

		if {"[$w.class${n}lab get]" != "" && \
		    "[$w.class${n}lab get]" != "(none)" } {
		    set labobj [mstclobj::labelTclObj #auto $classobj]
		    $labobj setAttr antialias         1
		    $labobj setAttr position          cc
		    $labobj setAttr minfeaturesize    auto
		    $labobj setAttr size              small
		    $labobj setAttr color             "255 255 255"
		    $labobj setAttr outlinecolor      "0 0 0"
		    $labobj setAttr backgroundcolor   "0 0 0"
		    $labobj setAttr minfeaturesize    auto
		}

		if {"[$w.class${n}sym get]" != "" && \
		    "[$w.class${n}sym get]" != "(none)" } {
		    $classobj setAttr symbol [$w.class${n}sym get]
		}
		
	    }
	}

        defineLayersGetLayer
        setMsg "layer \"$layname\" defined"
    } else {
        setMsg ""
    }

    destroy .layer
}
proc defineLayersOK {} {
    global layerOk
    set layerOk ok
}
proc defineLayersCANCEL {} {
    global layerOk
    set layerOk cancel
}
proc defineLayersDO {} {
    global nextCmd prevCmd mapobj
    set w .top.definelayers
    set layers ""
    foreach l [$mapobj getSubObjs] {
	if {"[$l getType]" == "layer"} {
	    lappend layers [$l getAttr name]
	}
    }
    
    if {[llength $layers] == 0} {
	appErr "No layers have been defined.\n\n\
                  Please define at least one layer for your application."
	return
    }
    setMsg ""
    defineWeb
}

#
# defineLayers
################################################################################

################################################################################
# defineWeb
#
proc defineWeb {} {
    global nextCmd prevCmd mapobj all_layers
    global html tmploc imgtype

    set initial_htdoc /usr/local/apache/htdocs/
    set initial_cgi   /cgi-bin/mapserv

    # read defaults
    if {[file isfile [file join ~ .msappwiz]]} {
	set fd [open [file join ~ .msappwiz]]
	set lines [read $fd]
        close $fd
        foreach {var linenum} {initial_htdoc 0 initial_cgi 1} {
	    set line [string trim [lindex [split $lines \n] $linenum]]
	    if {[string length $line]} {
		set $var $line
            }
        }
    }

    set w .top.defineweb
    if {![winfo exists $w]} {
	frame $w -width 550 -height 350
	label $w.m -text "\nWeb Application\n\nDefine attributes for the\
             web pages to be generated.\n"	
	set html plain
	frame $w.t
	label $w.t.l1 -text "Navigation:"
	radiobutton $w.t.type_html -text "HTML Form" \
			-variable html  -value plain
	radiobutton $w.t.type_jscr -text "JavaScript" \
			-variable html  -value javascript
	radiobutton $w.t.type_java -text "Java Applet" \
			-variable html  -value java
	pack $w.t.l1 $w.t.type_html $w.t.type_jscr $w.t.type_java -side left \
			-padx 4

	frame $w.h
	label $w.h.home_l -width 32 -anchor e \
			-text "Web server pages directory:"
	entry $w.h.home -width 35
	$w.h.home insert 0 $initial_htdoc
	pack $w.h.home_l $w.h.home -side left -padx 4

	frame $w.a
	label $w.a.apphome_l -width 32 -anchor e \
			     -text "Map application home directory,\n\
                                     relative to web server home:"
	entry $w.a.apphome -width 35
	$w.a.apphome insert 0 [$mapobj getAttr name]
	pack $w.a.apphome_l $w.a.apphome -side left -padx 4

	frame $w.c
	label $w.c.cgi_l -width 32 -anchor e \
			 -text "Mapserver CGI location:"
	entry $w.c.cgi -width 35
	$w.c.cgi insert 0 $initial_cgi
	pack $w.c.cgi_l $w.c.cgi -side left  -padx 4

        frame $w.g
        label $w.g.graph_l -text "Map image type:"
	set imgtype gif
        radiobutton $w.g.gif -text GIF  -variable imgtype -value gif 
        radiobutton $w.g.jpg -text JPG  -variable imgtype -value png
        radiobutton $w.g.png -text PNG  -variable imgtype -value jpg
        radiobutton $w.g.bmp -text WBMP -variable imgtype -value wbmp
	pack $w.g.graph_l $w.g.gif $w.g.jpg $w.g.png $w.g.bmp \
		-side left -padx 4

	frame $w.l
	label $w.l.tmp_l -text "Image tmp directory:"
	set tmploc common
	radiobutton $w.l.tmp_com -text "Common (/tmp)" \
		-variable tmploc -value common
	radiobutton $w.l.tmp_app -text "Application (\$apphome/tmp)" \
			-variable tmploc -value application
	pack $w.l.tmp_l $w.l.tmp_com $w.l.tmp_app  -side left -padx 4
	
	frame $w.i
        label $w.i.init_l -text "select layers\ninitially rendered:"
	scrollbox $w.i.init -scrollbar on -selectmode multiple \
		-exportselection no -height 5 -width 20
	pack $w.i.init_l $w.i.init -side left -padx 4 -pady 6

	pack $w.m $w.t $w.h $w.a $w.c $w.g $w.l $w.i -side top
    }
    pack forget [pack slaves .top]
    pack $w

    set all_layers ""
    foreach s [$mapobj getSubObjs] {
	if {"[$s getType]" == "layer"} {
	    lappend all_layers [$s getAttr name]
	}
    }
    set all_layers [lsort -unique $all_layers]
    foreach l $all_layers {
	$w.i.init insert end $l
    }

    set nextCmd defineWebDO
    set prevCmd defineLayers
}
proc defineWebDO {} {
    set w .top.defineweb
    global webserverhome mapapphome initial mapcgi

    set webserverhome [string trimright [string trim [$w.h.home get]]    /]
    set mapapphome    [string trimleft  [string trim [$w.a.apphome get]] /]
    set mapcgi        [string trim [$w.c.cgi get]]

    set initial ""
    foreach idx [$w.i.init curselection] {
        lappend initial [$w.i.init get $idx]
    }
    if {! [string length $initial]} {
	lappend initial [$w.i.init get 0]
    }

    # save webserverhome and mapcgi for next time
    catch {
	set fd [open [file join ~ .msappwiz] w]
	puts $fd $webserverhome
	puts $fd $mapcgi
        close $fd
    }

    generate
}

#
# defineWeb
################################################################################

################################################################################
# generate
#
proc generate {} {
    global nextCmd prevCmd mapobj
    global html tmploc imgtype
    set w .top.generate
    if {![winfo exists $w]} {
	frame $w -width 550 -height 350
	label $w.m -text "\nGenerate Map Application\n\nYou are almost\
             finished!\n\nPress \"Next\" to generate your mapfile and html\
             files.\n\n"	

	pack $w.m  -side top
    }
    pack forget [pack slaves .top]
    pack $w

    set nextCmd generateDO
    set prevCmd defineWeb
    .bot.next configure -text Next
    .bot.can  configure -state active
}
proc generateDO {} {
    global mapobj startupdir mapfile webserverhome mapapphome 
    global html tmploc tmpdir imgtype 

    set w .top.generate

    set mapname [$mapobj getAttr name]

    # fill in remaining attributes
    $mapobj setAttr shapepath data
    $mapobj setAttr status on
    $mapobj setAttr size  {600 600}
    $mapobj setAttr symbolset symbols/all.sym
    $mapobj setAttr imagecolor {255 255 255}
    if {"$imgtype" != "gif"} {
        $map setAttr imagetype $imgtype
    }

    # extents find greatest extent ranges in datafiles
    set minX 0.0
    set minY 0.0
    set maxX 0.0
    set maxY 0.0
    foreach subobj [$mapobj getSubObjs] {
	if {"[$subobj getType]" == "layer"} {
	    set df [$subobj getAttr data]
	    if {[file isfile data/$df.shp]} {
		set rc [mapscript::shapefileObj shp data/$df.shp -1]
        	if {"$rc" != "NULL"} {
		    set rectPtr [shp cget -bounds]
		    foreach v {minx miny maxx maxy} {
			set $v [mapscript::rectObjRef $rectPtr cget -$v]
		    }
		    rename shp {}
		    foreach {min v} {minX minx minY miny} {
			if {[set $min] == 0.0} {
			    set $min [set $v]
			} else {
			    if {[set $v] < [set $min]} {
				set $min [set $v]
			    }   
			}
		    }
		    foreach {max v} {maxX maxx maxY maxy} {
			if {[set $max] == 0.0} {
			    set $max [set $v]
			} else {
			    if {[set $v] > [set $max]} {
				set $max [set $v]
			    }   
			}
		    }
		}
	    }
	}
    }

    # set initial extents
    set map_extents "$minX $minY $maxX $maxY"
    $mapobj setAttr extent $map_extents

    
    # define a web object
    set webobj [mstclobj::webTclObj #auto $mapobj]
    $webobj setAttr template  ${mapname}.html
    $webobj setAttr header    ${mapname}_header.html
    $webobj setAttr footer    ${mapname}_footer.html
    if {"$tmploc" == "common"} {
        $webobj setAttr imagepath $webserverhome/tmp/
        $webobj setAttr imageurl  /tmp/
        set tmpdir "[string trimright $webserverhome /]/tmp"
    } else {
        $webobj setAttr imagepath $webserverhome/$mapapphome/tmp/
        $webobj setAttr imageurl  /$mapapphome/tmp/
        set tmpdir \
          "[string trimright $webserverhome /]/[string trim $mapapphome /]/tmp"
    }

    # define a querymap object
    set queryobj [mstclobj::querymapTclObj #auto $mapobj]
    $queryobj setAttr size "200 200"
    $queryobj setAttr status on
    $queryobj setAttr style hilite
    $queryobj setAttr color "255 0 0"

    # define a reference object
    set refobj [mstclobj::referenceTclObj #auto $mapobj]
    $refobj setAttr image "graphics/reference.$imgtype"
    $refobj setAttr extent $map_extents
    $refobj setAttr size "120 120"
    $refobj setAttr status on
    $refobj setAttr color "-1 -1 -1"
    $refobj setAttr outlinecolor "255 0 0"

    # define a legend object
    set legobj [mstclobj::legendTclObj #auto $mapobj]
    $legobj setAttr keysize "18 12"
    $legobj setAttr status on
    set labobj [mstclobj::labelTclObj #auto $legobj]
    $labobj setAttr type bitmap
    $labobj setAttr size medium
    $labobj setAttr color "0 0 89"


    set mapfile "[$mapobj getAttr name].map"
    $mapobj dumpToFile $mapfile

    # copy symbolset file from library
    file copy -force $startupdir/appwizlib/all.sym symbols/all.sym

    # copy symbolset images from library
    foreach f [glob -nocomplain $startupdir/appwizlib/images/*] {
	file copy -force $f symbols/images
    }
  
    # create an install script, which just copies map app directory to web  
    append install {#!/bin/sh}                                         \n  
    append install "webhome=$webserverhome"                            \n
    append install "apphome=$mapapphome"                               \n
    append install {if test ! -d $webhome/$apphome ; then}             \n
    append install {  mkdir $webhome/$apphome}                         \n
    append install {  mkdir $webhome/$apphome/data}                    \n
    append install {  mkdir $webhome/$apphome/images}                  \n
    append install {  mkdir $webhome/$apphome/graphics}                \n
    append install {  mkdir $webhome/$apphome/tmp}                     \n
    append install {  mkdir $webhome/$apphome/symbols}                 \n
    append install {  mkdir $webhome/$apphome/symbols/images}          \n
    append install {  chmod 755 $webhome/$apphome/data}                \n 
    append install {  chmod 755 $webhome/$apphome/images}              \n
    append install {  chmod 755 $webhome/$apphome/graphics}            \n
    append install {  chmod 755 $webhome/$apphome/tmp}                 \n 
    append install {  chmod 755 $webhome/$apphome/symbols}             \n 
    append install {  chmod 755 $webhome/$apphome/symbols/images}      \n 
    append install {fi}                                                \n
    append install {if test ! -d $webhome/tmp ; then}                  \n
    append install {  mkdir $webhome/tmp}                              \n
    append install {  chmod 775 $webhome/tmp}                          \n
    append install {fi}                                                \n
    append install {cp -r *.html *.map graphics data symbols \
                                                    $webhome/$apphome} \n
    append install {chmod 644 $webhome/$apphome/*.html}                \n
    append install {chmod 644 $webhome/$apphome/graphics/*}            \n

    set fd [open install.sh w]
    puts $fd $install
    close $fd

    generateHTML
    drawRefMap
    reviewMap
}

proc drawRefMap {} {
    global mapobj mapfile imgtype
    set map [mapscript::mapObj -args $mapfile]

    if {$map == "NULL"} {
	appErr "Cannot draw reference map image.\nFailed to create mapObj."
	catch {rename $map {}}
	return
    }

    # get the reference map size, and set map size to reference size
    set refPtr [$map cget -reference]
    set map_x [mapscript::referenceMapObjRef $refPtr cget -width]
    set map_y [mapscript::referenceMapObjRef $refPtr cget -height]
    $map configure -width  $map_x
    $map configure -height $map_y
    
    set img [$map prepareImage]

    # get the first layer
    set layer [$map getLayer 0]
    
    # turn off labels for this layer by fooling the label scale
    mapscript::layerObjRef $layer configure -labelmaxscale 0.0001

    # draw it
    mapscript::layerObjRef $layer draw $map $img
  
    # save to reference graphic
    mapscript::msSaveImage $img graphics/reference.$imgtype 1 1
    mapscript::msFreeImage $img
    rename $map {}
}

proc generateHTML {} {
    global mapobj startupdir querydefs webserverhome mapapphome 
    global html tmpdir imgtype initial all_layers mapcgi

    set mapname [$mapobj getAttr name]

    # generate query headers, footers & tables
    foreach query $querydefs {

	foreach {layerobj queryobj fieldlist} $query {break}
        # write the header
	set fd1 [open [$layerobj getAttr header] w]
	set fd2 [open [$queryobj getAttr template] w]
	set lay_name [$layerobj getAttr name]
	puts $fd1 "<font size+1><b>Layer: $lay_name</b></font><p>"
	puts $fd1 "<table cellpadding=5 cellspacing=2 border=0>"
	puts $fd1 "<tr bgcolor=#CCCCCC>"
	puts $fd2 "<tr>"
        foreach field $fieldlist {
	    puts $fd1 "<th>$field</th>"
	    puts $fd2 "<td>\[$field\]</td>"
	}
	puts $fd1 "</tr>"
	puts $fd2 "</tr>"
	close $fd1
	close $fd2

        # write the footer
        set fd [open [$layerobj getAttr footer] w]
	puts $fd "</table><p>"
	close $fd

    }
  
    set initiallayers ""
    foreach lay_name $initial {
	append initiallayers \
		"<input type='hidden' name='layer' value='$lay_name'>" \n
    }

    set layeroptions ""
    foreach lay_name [lsort -unique $all_layers] {
	append layeroptions \
	    "<option value='$lay_name' \[${lay_name}_select\]> $lay_name" \n
    }


    # generate index.html, map.html, map_footer.html map_header.html

    foreach {model_pg output_pg} [list model_init.html index.html \
	model.html $mapname.html model_footer.html ${mapname}_footer.html \
                                 model_header.html ${mapname}_header.html] {

        set fd [open $startupdir/appwizlib/$html/$model_pg]
        set model [read $fd]
        close $fd
        set fd [open $output_pg w]
        puts $fd [subst -nocommands -nobackslashes $model]
        close $fd
    }
    
}

#
# generate
###############################################################################

################################################################################
# reviewMap

proc reviewMap {} {
    global nextCmd prevCmd mapobj
    global html
    set w .top.reviewmap
    if {![winfo exists $w]} {
	frame $w -width 550 -height 350
	label $w.m -text "\nMap Application Generated\n\nYou may now review\
             your map with the MapServer Explorer or\nedit your application\
             with the MapServer Editor.\n\nYour web server administrator\
             can use the 'install.sh' script to install your\napplication."	

	button $w.startex -text "Start MapServer Explorer" -command startMSex
	button $w.started -text "Start MapServer Editor"   -command startMSed
	label $w.regen_l -text "\nYou can re-generate your map with the\
                extents currently shown in MapServer Explorer.\nStart the\
                MapServer Explorer, pan and zoom to the initial extents you\n\
                wish, and press the \"Re-generate\" button."
        button $w.regen -text "Re-generate" -command regenerate

	pack $w.m $w.startex $w.started $w.regen_l $w.regen \
                     -side top -pady 6
    }
    pack forget [pack slaves .top]
    pack $w

    set nextCmd exit
    set prevCmd generate
    .bot.next configure -text Finish
    .bot.can  configure -state disabled
    .bot.prev configure -state disabled
}

#
# reviewMap
###############################################################################

proc mkMain {} {
   
    wm title . "MapServer App Wizard"
    wm protocol . WM_DELETE_WINDOW doCancel
    wm geom . 560x400

    frame .top -width 550 -height 350 -relief raised
    frame .bot -relief raised

    button .bot.prev -text Previous -command doPrevious -state disabled
    button .bot.next -text Next     -command doNext     
    button .bot.can  -text Cancel   -command doCancel

    pack .bot.can -side right -padx 10
    pack .bot.next -side right -padx 1
    pack .bot.prev -side right -padx 1

    pack .top -side top -expand 1 -fill both
    pack .bot -side top -fill x

    label .msg -anchor w
    pack .msg -side top -fill x -anchor w
    
}


proc chooseColor {win_r win_g win_b} {
    global editattrs 
    set r [$win_r get]
    set g [$win_g get]
    set b [$win_b get]
    if {[catch {set init [format #%02x%02x%02x $r $g $b]}]} {
	set init #808080
    } elseif {[string length $init] != 7} {
	set init #808080
    }
    set newcolor [tk_chooseColor -initialcolor $init -title "Choose Color"]
    if {[string length $newcolor]} {
        set r [string range $newcolor 1 2]
        set g [string range $newcolor 3 4]
        set b [string range $newcolor 5 6]
        $win_r delete 0 end
        $win_g delete 0 end
        $win_b delete 0 end
        $win_r insert 0 [format %d 0x$r]
        $win_g insert 0 [format %d 0x$g]
        $win_b insert 0 [format %d 0x$b]
    }
}


proc setMsg {txt} {
    .msg configure -text $txt
    update idletasks
}



proc parseWithMS {} {
    global mapobj mapfile  title

    if {[catch {package require Mapscript}]} {
	tk_messageBox -icon error -type ok -title $title \
            -message "Mapscript Tcl extension is not available.  "
	return
    }

    if {[string length $mapobj] == 0} {
	tk_messageBox -icon warning -type ok -title $title \
            -message "No mapfile is\
             loaded.  Open an existing mapfile in order to use Mapscript parser"
	return
    }
    if {[string length $mapfile] == 0} {
	tk_messageBox -icon warning -type ok -title $title \
            -message "You must save your\
             current map objects to a file before you can use MapServer parser"
	return
    }
    if {[string length $mapobj]} {
	set opt [tk_messageBox -icon warning -title $title \
             -type okcancel -message "Changes\
                have been made to mapfile.\n\nParse existing mapfile without\
                saving current mapfile objects?"]
	switch $opt {
	    ok {}
	    cancel {return}
	}
    }

    set mapParseObj [mapscript::mapObj -args $mapfile]
    if {"$mapParseObj" == "NULL"} {
        set msg "Mapfile [file tail $mapfile] has errors:\n\n"
        foreach v {-code -routine -message} {
           append msg "$v: [mapscript::ms_error cget $v]\n"
        }
    } else {
	set msg "Mapfile [file tail $mapfile] parsed successfully"
    }
    # destroy map object
    catch {rename $mapParseObj {}}

    tk_messageBox -icon info -type ok -title "Mapserver Parser" \
	-message $msg
    
}

proc regenerate {} {
    global mapobj title
    set exe_name msexplorer.tcl
    if {[lsearch [winfo interps] $exe_name] == -1} {
	appErr "You must first start MapServer Explorer."
	return
    }
    set map_extents [send $exe_name remoteGetExtents]
    $mapobj setAttr extent $map_extents
    set mapfile "[$mapobj getAttr name].map"
    $mapobj dumpToFile $mapfile
    drawRefMap
    tk_messageBox -title $title -type ok -message "Your\
           mapfile has been re-generated with current extents."
    startMSex
}

proc getInterps {} {
    global tcl_platform

    if {"$tcl_platform(platform)" == "windows"} {
	update
        set all_interps [dde services TclEval {}]
        set interps ""
        foreach intrp $all_interps {
            lappend interps [lindex $intrp 1]
        }
    } else {
        set interps [winfo interps]
    }
    return $interps
}


proc startMSExe {exe_name} {
    global mapobj mapfile startupdir tcl_platform title
    if {[lsearch [getInterps] $exe_name] == -1} {
        if {"$tcl_platform(platform)" == "windows"} {
            set start_exe [list exec start [file join $startupdir $exe_name] &]
            set error_msg "Cannot 'send' to $exe_name via DDE emulation."
        } else {
            set start_exe [list exec [file join $startupdir $exe_name] &]
            set error_msg "Cannot 'send' to $exe_name. Check your xhost\
                settings, xhost should report\
                \n\"access control enabled, only authorized clients can\
                 connect\" "
        }
        if {[catch $start_exe result]} {
            tk_messageBox -icon error -type ok \
                -title $title \
                -message "Cannot start $exe_name\n\n$result"
            return 0
        } else {
            setMsg "please wait, starting $exe_name"
            # wait for Tk program to startup
            set i 20
            while {$i >= 0 && [lsearch [getInterps] $exe_name] == -1} {
                update
                after 1000
                incr i -1
                update
            }
            if {$i <= 0 && [lsearch [getInterps] $exe_name] == -1} {
                tk_messageBox -icon error -type ok \
                     -title $title \
                     -message "$error_msg\n\n$result"
                setMsg "could not contact $exe_name"
                return 0
            }
            after 1000
        }
    }
    update
    if {[catch {send $exe_name update} result]} {
        tk_messageBox -icon error -type ok -title $title \
                     -message "$error_msg\n\n$result"
        return 0
    }

    return 1
}


proc startMShapex {shpfile} {
    global mapobj mapfile startupdir title
    set exe_name msshapex.tcl

    if {! [startMSExe $exe_name]} {
        return
    }

    set mfile [file join [pwd] [file tail $shpfile]]
    update
    if {[catch {send $exe_name remoteOpenFile $mfile} result]} {
        tk_messageBox -icon error -type ok -title $title \
                     -message "Cannot 'send' to $exe_name\n\n$result"
    }
}

proc startMSex {} {
    global mapobj mapfile startupdir title
    set exe_name msexplorer.tcl
    if {! [startMSExe $exe_name]} {
        return
    }
    set mfile [file join [pwd] [file tail $mapfile]]
    update
    if {[catch {send $exe_name remoteOpenFile $mfile} result]} {
	tk_messageBox -icon error -type ok -title $title \
		     -message "Cannot 'send' to $exe_name\n\n$result"
    }
}

proc startMSed {} {
    global mapfile startupdir title
    set exe_name msedit.tcl

    if {! [startMSExe $exe_name]} {
        return
    }
    set mfile [file join [pwd] [file tail $mapfile]]
    update
    if {[catch {send $exe_name remoteOpenFile $mfile} result]} {
        tk_messageBox -icon error -type ok -title $title \
                     -message "Cannot 'send' to $exe_name\n\n$result"
    }
}


############################################################################
# 'send' emulation for windows
#
# written by George A. Howlett, 
# scarfed from a comp.lang.tcl posting by George.
#
#

# --------------------------------------------------------------------------
#
# SendInit --
#	
#	Creates a "send" proc to replace the former Tk send command.
#	Uses DDE services to simulate the transfer.  This must be
#	called before any drag&drop targets are registered. Otherwise
#	they will pick up the wrong application name.
#
#	The first trick is to determine a unique application name. This
#	is what other applications will use to send to us.  Tk used to
#	do this for us.  
#	
#	Note that we can generate the same name for two different Tk 
#	applications.  This can happen if two Tk applications picking
#	names at exactly the same time.   [In the future, we should 
#	probably generate a name based upon a global system value, such
#	as the handle of the main window "."]   The proc "SendVerify"
#	below will verify that you have only one DDE server registered
#	with this application's name.
#	
# Arguments:
#	myInterp	Sets the application name explicitly to this 
#			string. If the argument isn't given, or is the
#			empty string, then the routine picks a name for
#			us.
#
# Results:
#	Returns the name of the application.
#
# Side Effects:
#	Sets the name of our application.  You can call "tk appname" to
#	get the name.  A DDE topic using the same name is also created.
#	A send proc is also automatically created.  Be careful that you
#	don't overwrite an existing send command.
#
# --------------------------------------------------------------------------

proc SendInit { {myInterp ""} } {

    # Load the DDE package.
    package require dde

    if { $myInterp == "" } {

	# Pick a unique application name, replicating what Tk used to do.
	# This is what other applications will use to "send" to us. We'll
	# use DDE topics to represent interpreters.

	set appName [tk appname]
	set count 0
	set suffix {}
	
	# Keep generating interpreter names by suffix-ing the original
	# application name with " #number".  Sooner of later we'll find
	# one that's not currently use. 

	while { 1 } {
	    set myInterp "${appName}${suffix}"
	    set myServer [list TclEval $myInterp]
	    if { [lsearch [dde services TclEval {}] $myServer] < 0 } {
		break
	    }
	    incr count
	    set suffix " \#$count"
	}
    }
    tk appname $myInterp
    dde servername $myInterp
    proc send { interp args } {
	dde eval $interp $args
    }
    return $myInterp
}


# --------------------------------------------------------------------------
#
# SendVerify --
#	
#	Verifies that application name picked is uniquely registered
#	as a DDE server.  This checks that two Tk applications don't
#	accidently use the same name.
#
# Arguments:
#	None		Used the current application name.
#
# Results:
#	Generates an error if either a server can't be found or more
#	than one server is registered.
#
# --------------------------------------------------------------------------

proc SendVerify {} {
    # Load the DDE package.
    package require dde

    set count 0
    set appName [tk appname]
    foreach server [dde services TclEval {}] {
	set topic [lindex $server 1]
	if { [string compare $topic $appName] == 0 } {
	    incr count
	}
    }
    if {$count == 0} {
	error "Service not found: wrong name registered???"
    } 
    if { $count > 1 } {
	error "Duplicate names found for \"[tk appname]\""
    } 
}

#
###########################################################################


# main code


if {"$tcl_platform(platform)" == "windows"} {
    set scriptname [file tail scriptname]
    SendInit $scriptname
    if {[catch {SendVerify} reason]} {
        tk_messageBox -icon error -type warning \
                -title $title \
                -message "Cannot initialize \"send\" emulation using DDE.\n\
                    \n$reason\n\
                    \nInteraction with other MapServer Workbench tools\
                    may not function properly."
    }
}


mkMain
loadSymbols
welcome


