
#
# ObsTcl: Proof that a flexible language is not as good as a good one.
#
# Copyright (c) 1998 The Regents of the University of California.
#       All Rights Reserved.  See the COPYRIGHT file for details.
#

# ObsTcl 2.0  modifcations by Tom Poindexter
# Copyright 1998 Tom Poindexter
#       All Rights Reserved.  See the COPYRIGHT file for details.
#
# Tom's corollary:  A flexible language *is* a good language.
#
#  - automatic object naming #auto
#  - toplevel classess always inherit from ::obstcl::Object
#  - inherit checks that it must be used first in a class directive
#  - ::obstcl::Object introspection methods 'class' 'this' 'isa'
#  - class introspection command ::obstcl::classInfo
#  - clean up -option handling
#  - add arrayvariable to class parser, allows initialization of array vars
#  - add arraycommon to class parser, allows initialization of class array vars
#



package provide obstcl 2.0

namespace eval ::obstcl {
    set ::obstcl::classlist {}
}

proc ::obstcl::class {class body} {
    set class ::[string trimleft $class :]
    catch {namespace delete $class}
    namespace eval $class {
	variable _subclasses_ ""
	variable _superclass_ ""
	variable _auto_ 0
	namespace export *
    }
    ::obstcl::parser::_parse_ $class $body
    lappend ::obstcl::classlist $class
    set classproc {
	proc CLASS {obj args} {
	    # tp: add auto object naming 
	    if {"$obj" == "#auto"} {
	        set n [incr CLASS::_auto_]
	        set obj "CLASS_${n}_"
	    }
	    set ns ::[string trimleft $obj :]
	    namespace eval $ns {
		namespace import -force CLASS::*
		foreach {_o_ _d_} [array get CLASS::_opts_] {
		    set _opts_($_o_) $_d_
		}
		foreach {_v_ _d_} [array get CLASS::_vinit_] {
		    set $_v_ $_d_
		}
		foreach {_v_ _d_} [array get CLASS::_arrvar_] {
		    array set $_v_ $_d_
		}
		catch {unset _o_ _v_ _d_}
		set _class_ CLASS
	    }
	    set objprocbody "namespace eval CLASS \$method $ns \$args"
	    uplevel [list proc $obj {method args} $objprocbody]
	    uplevel $obj constructor $args
	    return $obj
	}
    }
    regsub -all CLASS $classproc $class classproc
    uplevel $classproc
}

proc ::obstcl::delete {name} {
    $name destructor
    namespace delete ::[string trimleft $name :]
    rename $name {}
}

namespace eval ::obstcl::parser {

    # tp: automatically inherit from ::obstcl::Object
    ::proc _checkInherit_ {} {
	::variable cls
	::variable inheritClass
	if {"$inheritClass" == "" && "$cls" != "::obstcl::Object"} {
	    uplevel 1 inherit ::obstcl::Object
	}
    }

    ::proc _parse_ {nm body} {
	::variable cls $nm
	::variable inheritClass ""
	set ${cls}::_vupvar_ {}
	set ${cls}::_cupvar_ {}
	eval $body
	array set temp [set ${cls}::_cupvar_]
	set commons {}
	foreach {r n} [array get temp] {
	    lappend commons $n $r
	}
	set ${cls}::_methhdr_ "upvar [set ${cls}::_vupvar_] $commons\n"
	foreach {name argv} [array get ${cls}::_meths_] {
	    uplevel 2 [list proc ${cls}::$name "this $argv" \
		    "[set ${cls}::_methhdr_]\n [set ${cls}::_body_($name)]"]
	}
	set ${cls}::_prochdr_ "upvar ${cls}::___ ___ $commons\n"
	foreach {name argv} [array get ${cls}::_procs_] {
	    uplevel 2 [list proc ${cls}::$name $argv \
		    "[set ${cls}::_prochdr_]\n [set ${cls}::_body_($name)]"]
	}
    }
    ::proc common {name {default UNDEF}} {
	::variable cls
	_checkInherit_
	if { $default != "UNDEF" } {
	    set ${cls}::$name $default
	}
	lappend ${cls}::_cupvar_ $name ${cls}::$name
    }
    ::proc arraycommon {name {default {}}} {
	::variable cls
	_checkInherit_
	array set ${cls}::$name $default
	lappend ${cls}::_cupvar_ $name ${cls}::$name
    }
    ::proc constructor {argv body} {
	::variable cls
	_checkInherit_
	if { [set ${cls}::_superclass_] != "" } {
	    set body \
		    "eval [set ${cls}::_superclass_]::constructor \$this\n \
		    $body"
	}
	set ${cls}::_meths_(constructor) $argv
	set ${cls}::_body_(constructor) $body
    }
    ::proc destructor {body} {
	::variable cls
	_checkInherit_
	if {[set ${cls}::_superclass_] != ""} {
	    set body "$body\n \
		    eval [set ${cls}::_superclass_]::destructor \$this"
	}
	set ${cls}::_meths_(destructor) {}
	set ${cls}::_body_(destructor) $body
    }
    ::proc inherit {super} {
	::variable cls
	::variable inheritClass
        if {"$inheritClass" != ""} {
	    error "must inherit before other class directives"
	}
	set inheritClass $super
	set super ::[string trimleft $super :]
	if { [namespace which -command $super] == "" } {
	    set temp $cls;
	    if ![auto_load $super] {
		error "Unknown class: \"$super\""
	    }
	    set cls $temp
	}
	namespace eval $cls  "namespace import -force ${super}::*"
	namespace eval $super [list lappend _subclasses_ $cls]
	set ${cls}::_superclass_ $super
	set ${cls}::_vupvar_ [set ${super}::_vupvar_]
	set ${cls}::_cupvar_ [set ${super}::_cupvar_]
	array set ${cls}::_opts_   [array get ${super}::_opts_]
	array set ${cls}::_config_ [array get ${super}::_config_]
	array set ${cls}::_vinit_  [array get ${super}::_vinit_]
	array set ${cls}::_arrvar_ [array get ${super}::_arrvar_]
    }
    ::proc method {name argv {body {}}} {
	::variable cls
	_checkInherit_
	set ${cls}::_meths_($name) $argv
	set ${cls}::_body_($name) $body
    }
    ::proc option {name {default UNDEF} {config {}}} {
	::variable cls
	_checkInherit_
	set ${cls}::_opts_($name) $default
	set ${cls}::_config_($name) $config
	append ${cls}::_vupvar_ "\$\{this\}::_opts_($name) $name "
    }
    ::proc proc {name argv {body {}}} {
	::variable cls
	_checkInherit_
	set ${cls}::_procs_($name) $argv
	set ${cls}::_body_($name) $body
    }
    ::proc variable {name {default UNDEF}} {
	::variable cls
	_checkInherit_
	if { $default != "UNDEF" } {
	    set ${cls}::_vinit_($name) $default
	}
	append ${cls}::_vupvar_ "\$\{this\}::$name $name "
    }
    ::proc arrayvariable {name {default {}}} {
	::variable cls
	_checkInherit_
	set ${cls}::_arrvar_($name) $default
	append ${cls}::_vupvar_ "\$\{this\}::$name $name "
    }
}


#
# The obstcl class ::Object
#
# Copyright (c) 1998 The Regents of the University of California.
#       All Rights Reserved.  See the COPYRIGHT file for details.
#
proc ::obstcl::Object {} {} ;# for auto_mkindex

::obstcl::class ::obstcl::Object {
    proc _checkopt {o} {
	if ![info exists ${_class_}::_opts_($o)] {
	    error "Unknown option: \"$o\""
	}
    }
    method cget {option} {
	_checkopt $this $option
	set ${this}::_opts_($option)
    }
    method configure {{option {}} args} {
	if { $option == "" } {
	    set o $option
	    set result {}
	    foreach {o v} [array get ${_class_}::_opts_] {
		lappend result [list $o $v [set ${this}::_opts_($o)]]
	    }
	    return $result
	} elseif {$args == ""} {
	    set o $option
	    _checkopt $this $o
	    return [list $option \
		    [set ${_class_}::_opts_($o)] \
		    [set ${this}::_opts_($o)]]
	} else {
	    foreach {o v} [linsert $args 0 $option] {
		_checkopt $this $o
		set ${this}::_opts_($o) $v
		if { [set ${_class_}::_config_($o)] != "" } {
		    eval [set ${_class_}::_methhdr_]
		    eval [set ${_class_}::_config_($o)]
		}
	    }
	}
    }
    method constructor {args} {}
    method destructor  {args} {}
    # Chain is special
    proc chain {args} {
	set object [uplevel set this]
	set class [uplevel namespace current]
	set method [namespace tail [lindex [info level -1] 0]]
	namespace eval [set [set class]::_superclass_] "$method $object $args"
    }
    # This is needed by class.tcl
    variable _class_ ""


    # tp: add class introspection methods for objects
    method class {} {
        return $_class_
    }
    method this {} {
	return $this
    }
    method isa {c} {
	set c "::[string trimleft $c :]"
	if {"$c" == "$_class_"} {
	    return 1
	} else {
	    set s $_class_
	    while {[catch {set s [set ${s}::_superclass_]}] == 0} {
	        if {"$c" == "$s"} {
		    return 1
	        }
	    }
	}
	return 0
    }

}


# tp: class introspection commands, stand alone

proc ::obstcl::_checkClass_ {class} {
    variable classlist
    set class ::[string trimleft $class :]
    if { [lsearch $classlist $class] == -1 } {
	error "$class is not defined"
    }
    return $class
}

proc ::obstcl::classInfo {cmd args} {

    switch -- $cmd {

	classlist {
            variable classlist
	    return $classlist
	}

	superclass {
	    set _class_ [_checkClass_ [lindex $args 0]]
	    return [set ${_class_}::_superclass_]
        }

	subclasses {
	    set _class_ [_checkClass_ [lindex $args 0]]
	    return [set ${_class_}::_subclasses_]
        }

        heritage {
	    set _class_ [_checkClass_ [lindex $args 0]]
	    lappend classList $_class_
	    set s $_class_
	    while {[catch {set s [set ${s}::_superclass_]}] == 0} {
		lappend classList $s
	    }
	    return $classList
	}

	methods {
	    set _class_ [_checkClass_ [lindex $args 0]]
	    set methList ""
	    foreach cls [classInfo heritage $_class_] {
		foreach {name args} [array get ${cls}::_meths_] {
		    if {[lsearch -exact $methList $name] == -1} {
			lappend methList $name
		    }
		}
	    }
	    return $methList
	}

	methodsByClass {
	    set _class_ [_checkClass_ [lindex $args 0]]
	    set methList ""
	    foreach cls [classInfo heritage $_class_] {
		foreach {name args} [array get ${cls}::_meths_] {
		    lappend methList [list $cls $name]
		}
	    }
	    return $methList
	}

	methodArgs {
	    set _class_ [_checkClass_ [lindex $args 0]]
	    set m [lindex $args 1]
	    foreach cls [classInfo heritage $_class_] {
		if {[info exists ${cls}::_meths_($m)]} {
		    return [lrange [namespace eval $cls info args $m] 1 end]
		}
	    }
	    error "can't find method $m in $_class_"
	}

    }

}




